/*!@file
  \brief   APC main driver
  \details The driver assumes the existence of a bison-generated parser,
           referenced by the external function 'yyparse'.
           It also assumes the existence of a lexer which must be initialized
           before parsing, referenced by the external function 'lexer_init'
           which assumes standard error handling.
           All input arguments are made available through the exposed (that is,
           non-static) array of character pointers 'cargs', which point
           to the non-duplicated strings in 'argv' directly from the system.
  \author  Jordan Lavatai
  \date    Aug 2016
  ----------------------------------------------------------------------------*/
/* Standard */
#include <stdio.h>  //print
#include <errno.h>  //errors
#include <string.h> //strndupa
/* Posix */
#include <stdlib.h> //exit
#include <unistd.h> //getopt, sysconf
/* Internal */
#include "parser.tab.h" //bison
#include "ir.h"

#define DEFAULT_PAGESIZE 4096
const char** cargs;
long  sys_pagesize;
  
int main(int, char*[]);

extern //lexer.c
int  lexer_init(void);
extern //scanner.c
int  scanner_init(void);
extern //scanner.c
void scanner_quit(void);
extern //scanner.c
int  scanner_scanpath(char const*);
extern //ir.c
int  ir_init(void);
extern //ir.c
int  ir_linker(void);
extern //ir.c
int  ir_condenser(void);
extern
void     ir_test(void);

/* Main entry from terminal
   parses the command line and kicks off recursive scanning
*/
int main
( int   argc,
  char* argv[]
)
#define $($)#$ //stringifier
#define MAXSTR 255
#define MAXERR "-%c allows at most " $(MAXSTR) " input characters\n", opt
#define OPTS  "d:o:h-"
#define USAGE "Usage %s [-d dir_root][-o output_file][-h]\n", argv[0]
#define USAGE_LONG					\
  "\tOptions:\n"					\
  "\t\t-d\tRoot directory to parse from \t[./]\n"	\
  "\t\t-o\tOutput filename \t\t[a.asspak]\n"		\
  "\t\t-h\tPrint this help\n"
#define DONE   -1
#define SCANPATH (cargs['d'] ? cargs['d'] : "./")
{ int   opt;
  cargs = (const char**) malloc('Z');
 getopt:
  switch (opt = getopt(argc, argv, OPTS))
  { case 'd' :
    case 'o' :
      if (strnlen(optarg, MAXSTR) != MAXSTR)
        { cargs[opt] = optarg;
          goto getopt;
        }
      fprintf(stderr, MAXERR);
    default :
      fprintf(stderr, USAGE);
      exit(EXIT_FAILURE);
    case 'h' :
      printf(USAGE);
      printf(USAGE_LONG);
      exit(EXIT_SUCCESS);
    case DONE:
      break;
  }
  if ((sys_pagesize = sysconf(_SC_PAGESIZE)) == 0)
    sys_pagesize = DEFAULT_PAGESIZE;
  if (ir_init())
    { perror("init");
      free(cargs);
      exit(EXIT_FAILURE);
    }
  if (scanner_scanpath(SCANPATH))
    { perror("scanner");
      free(cargs);
      exit(EXIT_FAILURE);
    }
  ir_test();
  ir_linker();
  ir_condenser();
  free(cargs);
  exit(EXIT_SUCCESS);
}
